use std::path::{Path, PathBuf};
use std::{fs, io};

pub fn copy_recursively(source: impl AsRef<Path>, destination: impl AsRef<Path>) -> io::Result<()> {
    // Credit: https://nick.groenen.me/notes/recursively-copy-files-in-rust/
    fs::create_dir_all(&destination)?;
    for entry in fs::read_dir(source)? {
        let entry = entry?;
        let filetype = entry.file_type()?;
        if filetype.is_dir() {
            copy_recursively(entry.path(), destination.as_ref().join(entry.file_name()))?;
        } else {
            fs::copy(entry.path(), destination.as_ref().join(entry.file_name()))?;
        }
    }
    Ok(())
}

pub fn write_file(filename: PathBuf, html: String) {
    // copy all files in static to public
    fs::write(filename, html).expect("Could not write to index.html");
}


pub fn delete_output_dir(base: &PathBuf) {
    let root_dir = format!("./{}", base.display());
    if Path::new(&root_dir).exists() {
        fs::remove_dir_all(&root_dir).expect("could not remove directory");
    }
}


pub fn resolve_and_handle_path(current_dir: &Path, path_to_resolve: &Path) -> Option<PathBuf> {
    // Use the current directory as the base to resolve the relative path

    // Construct an absolute path by joining the current directory with the provided path
    let absolute_path = current_dir.join(path_to_resolve);

    // Canonicalize the absolute path
    if let Ok(canonicalized_path) = fs::canonicalize(&absolute_path) {
        // Path was successfully canonicalized
        println!("Canonicalized path: {:?}", canonicalized_path);
        Some(canonicalized_path)
    } else {
        // Path couldn't be canonicalized, create the directory
        if let Err(err) = fs::create_dir_all(&absolute_path) {
            eprintln!("Error creating directory: {:?}", err);
            None
        } else {
            println!("Directory created successfully!");
            Some(absolute_path)
        }
    }
}
